@@ -55,6 +55,7 @@ from The Open Group.
 #include "Xlibint.h"
 #include <X11/Xresource.h>
 #include "Xresinternal.h"
+#include "reallocarray.h"
 
 /* Not cost effective, at least for vanilla MIT clients */
 /* #define PERMQ */
@@ -62,7 +63,7 @@ from The Open Group.
 #ifdef PERMQ
 typedef unsigned char Bits;
 #endif
-typedef unsigned long Entry; /* dont confuse with EntryRec from Xintatom.h */
+typedef unsigned long Entry; /* don't confuse with EntryRec from Xintatom.h */
 
 static XrmQuark nextQuark = 1;	/* next available quark number */
 static unsigned long quarkMask = 0;
@@ -131,10 +132,8 @@ static char *permalloc(unsigned int length)
     return(ret);
 }
 
-#ifndef WORD64
 typedef struct {char a; double b;} TestType1;
 typedef struct {char a; unsigned long b;} TestType2;
-#endif
 
 #ifdef XTHREADS
 static char *_Xpermalloc(unsigned int length);
@@ -157,7 +156,6 @@ char *Xpermalloc(unsigned int length)
     int i;
 
     if (neverFreeTableSize && length < NEVERFREETABLESIZE) {
-#ifndef WORD64
 	if ((sizeof(TestType1) !=
 	     (sizeof(TestType2) - sizeof(unsigned long) + sizeof(double))) &&
 	    !(length & (DALIGN-1)) &&
@@ -165,7 +163,6 @@ char *Xpermalloc(unsigned int length)
 	    neverFreeTableSize -= DALIGN - i;
 	    neverFreeTable += DALIGN - i;
 	} else
-#endif
 	    if ((i = (NEVERFREETABLESIZE - neverFreeTableSize) & (WALIGN-1))) {
 		neverFreeTableSize -= WALIGN - i;
 		neverFreeTable += WALIGN - i;
@@ -190,15 +187,14 @@ ExpandQuarkTable(void)
 	newmask = (oldmask << 1) + 1;
     else {
 	if (!stringTable) {
-	    stringTable = (XrmString **)Xmalloc(sizeof(XrmString *) *
-						CHUNKPER);
+	    stringTable = Xmalloc(sizeof(XrmString *) * CHUNKPER);
 	    if (!stringTable)
 		return False;
 	    stringTable[0] = (XrmString *)NULL;
 	}
 #ifdef PERMQ
 	if (!permTable)
-	    permTable = (Bits **)Xmalloc(sizeof(Bits *) * CHUNKPER);
+	    permTable = Xmalloc(sizeof(Bits *) * CHUNKPER);
 	if (!permTable)
 	    return False;
 #endif
@@ -235,7 +231,7 @@ ExpandQuarkTable(void)
 	}
     }
     if (oldmask)
-	Xfree((char *)oldentries);
+	Xfree(oldentries);
     return True;
 }
 
@@ -247,7 +243,6 @@ _XrmInternalStringToQuark(
     register XrmQuark q;
     register Entry entry;
     register int idx, rehash;
-    register int i;
     register char *s1, *s2;
     char *new;
 
@@ -262,10 +257,11 @@ _XrmInternalStringToQuark(
 		goto nomatch;
 	    q = (entry >> QUARKSHIFT) & QUARKMASK;
 	}
-	for (i = len, s1 = (char *)name, s2 = NAME(q); --i >= 0; ) {
-	    if (*s1++ != *s2++)
+	s2 = NAME(q);
+	if(memcmp((char *)name, s2, len) != 0) {
 		goto nomatch;
 	}
+	s2 += len;
 	if (*s2) {
 nomatch:    if (!rehash)
 		rehash = REHASHVAL(sig);
@@ -293,15 +289,15 @@ nomatch:    if (!rehash)
     q = nextQuark;
     if (!(q & QUANTUMMASK)) {
 	if (!(q & CHUNKMASK)) {
-	    if (!(new = Xrealloc((char *)stringTable,
-				 sizeof(XrmString *) *
-				 ((q >> QUANTUMSHIFT) + CHUNKPER))))
+	    if (!(new = Xreallocarray(stringTable,
+                                      (q >> QUANTUMSHIFT) + CHUNKPER,
+                                      sizeof(XrmString *))))
 		goto fail;
 	    stringTable = (XrmString **)new;
 #ifdef PERMQ
-	    if (!(new = Xrealloc((char *)permTable,
-				 sizeof(Bits *) *
-				 ((q >> QUANTUMSHIFT) + CHUNKPER))))
+	    if (!(new = Xreallocarray(permTable,
+                                      (q >> QUANTUMSHIFT) + CHUNKPER,
+                                      sizeof(Bits *))))
 		goto fail;
 	    permTable = (Bits **)new;
 #endif
@@ -323,9 +319,9 @@ nomatch:    if (!rehash)
 #endif
 	if (!name)
 	    goto fail;
-	for (i = len, s1 = (char *)name; --i >= 0; )
-	    *s1++ = *s2++;
-	*s1++ = '\0';
+	s1 = (char*)name;
+	memcpy(s1, s2, (size_t)len);
+	s1[len] = '\0';
 #ifdef PERMQ
 	CLEARPERM(q);
     }
@@ -402,7 +398,7 @@ XrmString XrmQuarkToString(register XrmQuark quark)
     else {
 #ifdef PERMQ
 	/* We have to mark the quark as permanent, since the caller might hold
-	 * onto the string pointer forver.
+	 * onto the string pointer forever.
 	 */
 	SETPERM(quark);
 #endif
